//
//! \file  libecc/debug.h
//! \brief Debugging declarations and defines.
//!
//! This header file defines debugging macros
//! and declares debug channels that are being
//! used when libecc was configured with
//! \link enable_debug --enable-debug \endlink.
//!
//! \sa http://libcwd.sourceforge.net/reference-manual/group__chapter__custom__debug__h.html
//
// This file is part of the libecc package.
// Copyright (C) 2002, 2004 by
// 
// Carlo Wood, Run on IRC <carlo@alinoe.com>
// RSA-1024 0x624ACAD5 1997-01-26                    Sign & Encrypt
// Fingerprint16 = 32 EC A7 B6 AC DB 65 A6  F6 F6 55 DD 1C DC FF 61
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef LIBECC_DEBUG_H
#define LIBECC_DEBUG_H

#include "libecc/config.h"

#ifdef CWDEBUG
#include <libcwd/libraries_debug.h>

namespace libecc {
  namespace debug {
    namespace channels {
      namespace dc {
        using namespace libcwd::channels::dc;
        extern libcwd::channel_ct ecc;
        extern libcwd::channel_ct polynomial;
        extern libcwd::channel_ct bitsetfind1;
        extern libcwd::channel_ct bitsetshift;
	extern libcwd::channel_ct gaussj;
      }
    }
  }
}
#endif // CWDEBUG

#if ECC_DEBUG
// Define private debug output macros for use in header files of the library.
#define LibEccDebug(STATEMENT...) LibcwDebug(libecc::debug::channels, STATEMENT)
#define LibEccDout(cntrl, data) LibcwDout(libecc::debug::channels, libcwd::libcw_do, cntrl, data)
#define LibEccDoutFatal(cntrl, data) LibcwDoutFatal(libecc::debug::channels, libcwd::libcw_do, cntrl, data)
#define LibEccForAllDebugChannels(STATEMENT...) LibcwdForAllDebugChannels(libecc::debug::channels, STATEMENT)
#define LibEccForAllDebugObjects(STATEMENT...) LibcwdForAllDebugObjects(libecc::debug::channels, STATEMENT)
#else
#define LibEccDebug(STATEMENT...)
#define LibEccDout(cntrl, data)
#define LibEccDoutFatal(cntrl, data)
#define LibEccForAllDebugChannels(STATEMENT...)
#define LibEccForAllDebugObjects(STATEMENT...)
#endif

// All other macros might be used in header files of libecc, but need to be
// defined by the debug.h of the application that uses it.
// LIBECC_INTERNAL is defined when the library itself is being compiled (see below).
#if !defined(Debug) && !defined(LIBECC_INTERNAL)
#error The application source file (.cc or .cpp) must use '#include "debug.h"' _before_ including the header file that it includes now, that led to this error.
#endif

#endif // LIBECC_DEBUG_H
