#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestFixture.h>
#include <cppunit/TestSuite.h>
#include <cppunit/TestCaller.h>
#include <libecc/bitset.h>
#include <inttypes.h>

template<unsigned int n>
  class OverLoadHook { };

class bitsetTest : public CppUnit::TestFixture {
  static size_t const digit_size = sizeof(libecc::bitset_digit_t);
  static size_t const digit_bits = ECC_BITS;

  static char const* const b7v3_str;
  static char const* const b7v3_bare;
  static char const* const b7v3_fit;
  static char const* const b7v3_excess;

  static char const* const bDSv3_str;
  static char const* const bDSv3_bare;
  static char const* const bDSv3_excess1;
  static char const* const bDSv3_excess2;
  static char const* const bDSv3_excess3;
  static char const* const bDSv3_excess4;

  static char const* const bDSvDS_str;
  static char const* const bDSvDS_excess1;
  static char const* const bDSvDS_excess2;
  static char const* const bDSvDS_excess3;

  static char const* const bDSp7vDS_str;
  static char const* const bDSp7vDS_bare;
  static char const* const bDSp7vDS_fit;
  static char const* const bDSp7vDS_excess1;
  static char const* const bDSp7vDS_excess2;

  static char const* const b2DSv2DSm1_str;
  static char const* const b2DSv2DSm1_excess1;
  static char const* const b2DSv2DSm1_excess2;

  static char const* const b2DSv2DS_str;
  static char const* const b2DSv2DS_excess;

  static char const* const b2DSp1v2DSp1_str;
  static char const* const b2DSp1v2DSp1_fit;
  static char const* const b2DSp1v2DSp1_excess;

  static char const* const b6DSp3v6DSp3_str;
  static char const* const b6DSp3v6DSp3_fit;

  static char const* const b6DSp13v6DSp3_str;
  static char const* const b6DSp13v6DSp3_bare;
  static char const* const b6DSp13v6DSp3_fit;

  static char const* const b6DSp13v6DSp13_str;
  static char const* const b6DSp13v6DSp13_fit;

  enum
  {
    DS = ECC_BITS,
    DSp7 = ECC_BITS + 7,
    DSp13 = ECC_BITS + 13,
    twoDSm1 = 2 * ECC_BITS - 1,
    twoDS = 2 * ECC_BITS,
    twoDSp1 = 2 * ECC_BITS + 1,
    twoDSp13 = 2 * ECC_BITS + 13,
    threeDS = 3 * ECC_BITS,
    threeDSp13 = 3 * ECC_BITS + 13,
    fourDS = 4 * ECC_BITS,
    fourDSm1 = 4 * ECC_BITS - 1,
    fourDSp1 = 4 * ECC_BITS + 1,
    fourDSp2 = 4 * ECC_BITS + 2,
    fourDSp3 = 4 * ECC_BITS + 3,
    sixDSp3 = 6 * ECC_BITS + 3,
    sixDSp13 = 6 * ECC_BITS + 13
  };
  						// Bit pattern if DS == 32
  libecc::bitset<7> const b7v3;			//                                                                               0000101
  libecc::bitset<DS> const bDSv3;		//                                                      00000000000000000000000000000101
  libecc::bitset<DS> const bDSvDS;		//                                                      11010010001111011010111000000101
  libecc::bitset<DSp7> const bDSp7vDS;		//                                               000000011010010001111011010111000000101
  libecc::bitset<twoDS> const b2DSv2DS;		//                      1111100010100000000100111100011101000010001111011010111000000101
  libecc::bitset<twoDS> const b2DSv2DSm1;	//                      0100001000111101101011100000010111111000101000000001001111000111
  libecc::bitset<twoDSp1> const b2DSp1v2DSp1;	//                     11111100010100000000100111100011101000010001111011010111000000101
  libecc::bitset<sixDSp3> const b6DSp3v6DSp3;	//           10111...0001111100010100000000100111100011101000010001111011010111000000101
  libecc::bitset<sixDSp13> const b6DSp13v6DSp3;	// 000000000010111...0001111100010100000000100111100011101000010001111011010111000000101
  libecc::bitset<sixDSp13> const b6DSp13v6DSp13;// 110011100010111...0001111100010100000000100111100011101000010001111011010111000000101

  enum
  {
    eb7v3		=        7 * 1000,
    ebDSv3		=       DS * 1000 + 1,
    ebDSvDS		=       DS * 1000 + 2,
    ebDSp7vDS		=     DSp7 * 1000 + 3,
    eb2DSv2DS		=    twoDS * 1000 + 4,
    eb2DSv2DSm1		=    twoDS * 1000 + 5,
    eb2DSp1v2DSp1	=  twoDSp1 * 1000 + 6,
    eb6DSp3v6DSp3	=  sixDSp3 * 1000 + 7,
    eb6DSp13v6DSp3	= sixDSp13 * 1000 + 8,
    eb6DSp13v6DSp13	= sixDSp13 * 1000 + 9,
    last_ebv		=                   10
  };

  // Accessors for b.
  libecc::bitset<7> const& get_b(OverLoadHook<eb7v3>) { return b7v3; }
  libecc::bitset<DS> const& get_b(OverLoadHook<ebDSv3>) { return bDSv3; }
  libecc::bitset<DS> const& get_b(OverLoadHook<ebDSvDS>) { return bDSvDS; }
  libecc::bitset<DSp7> const& get_b(OverLoadHook<ebDSp7vDS>) { return bDSp7vDS; }
  libecc::bitset<twoDS> const& get_b(OverLoadHook<eb2DSv2DS>) { return b2DSv2DS; }
  libecc::bitset<twoDS> const& get_b(OverLoadHook<eb2DSv2DSm1>) { return b2DSv2DSm1; }
  libecc::bitset<twoDSp1> const& get_b(OverLoadHook<eb2DSp1v2DSp1>) { return b2DSp1v2DSp1; }
  libecc::bitset<sixDSp3> const& get_b(OverLoadHook<eb6DSp3v6DSp3>) { return b6DSp3v6DSp3; }
  libecc::bitset<sixDSp13> const& get_b(OverLoadHook<eb6DSp13v6DSp3>) { return b6DSp13v6DSp3; }
  libecc::bitset<sixDSp13> const& get_b(OverLoadHook<eb6DSp13v6DSp13>) { return b6DSp13v6DSp13; }

  static char const* const b_str[last_ebv];

  // 0110010110011010 = 659A
  // 0011001100111100 = 333C
  // -----------------------------
  // 0010000100011000 = 2118   AND
  // 0111011110111110 =        OR
  // 0101011010100110 =        XOR
  // -----------------------------
  // 1111010000001011 = F40B
  static char const* const x_str;
  static char const* const y_str;
  static char const* const z_str;
  static char const* const x_AND_y_13_str;
  static char const* const x_AND_y_DS_str;
  static char const* const x_AND_y_DSp13_str;
  static char const* const x_AND_y_2DS_str;
  static char const* const x_AND_y_2DSp13_str;
  static char const* const x_AND_y_3DS_str;
  static char const* const x_AND_y_3DSp13_str;
  static char const* const x_AND_y_4DS_str;

  libecc::bitset<13> const x13;
  libecc::bitset<13> const y13;
  libecc::bitset<13> const z13;
  libecc::bitset<DS> const xDS;
  libecc::bitset<DS> const yDS;
  libecc::bitset<DS> const zDS;
  libecc::bitset<DSp13> const xDSp13;
  libecc::bitset<DSp13> const yDSp13;
  libecc::bitset<DSp13> const zDSp13;
  libecc::bitset<twoDS> const x2DS;
  libecc::bitset<twoDS> const y2DS;
  libecc::bitset<twoDS> const z2DS;
  libecc::bitset<twoDSp13> const x2DSp13;
  libecc::bitset<twoDSp13> const y2DSp13;
  libecc::bitset<twoDSp13> const z2DSp13;
  libecc::bitset<threeDS> const x3DS;
  libecc::bitset<threeDS> const y3DS;
  libecc::bitset<threeDS> const z3DS;
  libecc::bitset<threeDSp13> const x3DSp13;
  libecc::bitset<threeDSp13> const y3DSp13;
  libecc::bitset<threeDSp13> const z3DSp13;
  libecc::bitset<fourDS> const x4DS;
  libecc::bitset<fourDS> const y4DS;
  libecc::bitset<fourDS> const z4DS;

  // Accessors for x and y.
  libecc::bitset<13> const& get_x(OverLoadHook<13>) const { return x13; }
  libecc::bitset<13> const& get_y(OverLoadHook<13>) const { return y13; }
  libecc::bitset<13> const& get_z(OverLoadHook<13>) const { return z13; }
  libecc::bitset<DS> const& get_x(OverLoadHook<DS>) const { return xDS; }
  libecc::bitset<DS> const& get_y(OverLoadHook<DS>) const { return yDS; }
  libecc::bitset<DS> const& get_z(OverLoadHook<DS>) const { return zDS; }
  libecc::bitset<DSp13> const& get_x(OverLoadHook<DSp13>) const { return xDSp13; }
  libecc::bitset<DSp13> const& get_y(OverLoadHook<DSp13>) const { return yDSp13; }
  libecc::bitset<DSp13> const& get_z(OverLoadHook<DSp13>) const { return zDSp13; }
  libecc::bitset<twoDS> const& get_x(OverLoadHook<twoDS>) const { return x2DS; }
  libecc::bitset<twoDS> const& get_y(OverLoadHook<twoDS>) const { return y2DS; }
  libecc::bitset<twoDS> const& get_z(OverLoadHook<twoDS>) const { return z2DS; }
  libecc::bitset<twoDSp13> const& get_x(OverLoadHook<twoDSp13>) const { return x2DSp13; }
  libecc::bitset<twoDSp13> const& get_y(OverLoadHook<twoDSp13>) const { return y2DSp13; }
  libecc::bitset<twoDSp13> const& get_z(OverLoadHook<twoDSp13>) const { return z2DSp13; }
  libecc::bitset<threeDS> const& get_x(OverLoadHook<threeDS>) const { return x3DS; }
  libecc::bitset<threeDS> const& get_y(OverLoadHook<threeDS>) const { return y3DS; }
  libecc::bitset<threeDS> const& get_z(OverLoadHook<threeDS>) const { return z3DS; }
  libecc::bitset<threeDSp13> const& get_x(OverLoadHook<threeDSp13>) const { return x3DSp13; }
  libecc::bitset<threeDSp13> const& get_y(OverLoadHook<threeDSp13>) const { return y3DSp13; }
  libecc::bitset<threeDSp13> const& get_z(OverLoadHook<threeDSp13>) const { return z3DSp13; }
  libecc::bitset<fourDS> const& get_x(OverLoadHook<fourDS>) const { return x4DS; }
  libecc::bitset<fourDS> const& get_y(OverLoadHook<fourDS>) const { return y4DS; }
  libecc::bitset<fourDS> const& get_z(OverLoadHook<fourDS>) const { return z4DS; }

  libecc::bitset<13> const AND13;
  libecc::bitset<DS> const ANDDS;
  libecc::bitset<DSp13> const ANDDSp13;
  libecc::bitset<twoDS> const AND2DS;
  libecc::bitset<twoDSp13> const AND2DSp13;
  libecc::bitset<threeDS> const AND3DS;
  libecc::bitset<threeDSp13> const AND3DSp13;
  libecc::bitset<fourDS> const AND4DS;

  // Accessors for AND.
  libecc::bitset<13> const& get_AND(OverLoadHook<13>) const { return AND13; }
  libecc::bitset<DS> const& get_AND(OverLoadHook<DS>) const { return ANDDS; }
  libecc::bitset<DSp13> const& get_AND(OverLoadHook<DSp13>) const { return ANDDSp13; }
  libecc::bitset<twoDS> const& get_AND(OverLoadHook<twoDS>) const { return AND2DS; }
  libecc::bitset<twoDSp13> const& get_AND(OverLoadHook<twoDSp13>) const { return AND2DSp13; }
  libecc::bitset<threeDS> const& get_AND(OverLoadHook<threeDS>) const { return AND3DS; }
  libecc::bitset<threeDSp13> const& get_AND(OverLoadHook<threeDSp13>) const { return AND3DSp13; }
  libecc::bitset<fourDS> const& get_AND(OverLoadHook<fourDS>) const { return AND4DS; }

public:
  bitsetTest(void) : b7v3(std::string(b7v3_str)),
                      bDSv3(bDSv3_str),
		      bDSvDS(bDSvDS_str),
                      bDSp7vDS(bDSp7vDS_str),
		      b2DSv2DS(b2DSv2DS_str),
		      b2DSv2DSm1(b2DSv2DSm1_str),
		      b2DSp1v2DSp1(b2DSp1v2DSp1_str),
		      b6DSp3v6DSp3(b6DSp3v6DSp3_str),
		      b6DSp13v6DSp3(b6DSp13v6DSp3_str),
		      b6DSp13v6DSp13(b6DSp13v6DSp13_str),
		      x13(x_str),
		      y13(y_str),
		      z13(z_str),
		      xDS(x_str),
		      yDS(y_str),
		      zDS(z_str),
		      xDSp13(x_str),
		      yDSp13(y_str),
		      zDSp13(z_str),
		      x2DS(x_str),
		      y2DS(y_str),
		      z2DS(z_str),
		      x2DSp13(x_str),
		      y2DSp13(y_str),
		      z2DSp13(z_str),
		      x3DS(x_str),
		      y3DS(y_str),
		      z3DS(z_str),
		      x3DSp13(x_str),
		      y3DSp13(y_str),
		      z3DSp13(z_str),
		      x4DS(x_str),
		      y4DS(y_str),
		      z4DS(z_str),
		      AND13(x_AND_y_13_str),
		      ANDDS(x_AND_y_DS_str),
		      ANDDSp13(x_AND_y_DSp13_str),
		      AND2DS(x_AND_y_2DS_str),
		      AND2DSp13(x_AND_y_2DSp13_str),
		      AND3DS(x_AND_y_3DS_str),
		      AND3DSp13(x_AND_y_3DSp13_str),
		      AND4DS(x_AND_y_4DS_str) { }

  void testConsts(void);
  template<unsigned int n>
    void testOutputLevel2(void);
  void testOutput(void);
  template<unsigned int n>
    void testConstructionLevel2(char const* str);
  void testConstruction(void);
  void testEquality(void);
  void testInequality(void);
  template<unsigned int n>
    void testAssignmentLevel2(void);
  void testAssignment(void);
  template<unsigned int n>
    void testCopyLevel2(void);
  void testCopy(void);
  template<unsigned int n, unsigned int n13, unsigned int nDS, unsigned int nDSp13, unsigned int n2DS, unsigned int n2DSp13, unsigned int n3DS, unsigned int n3DSp13, unsigned int n4DS>
    void testAndAssignLevel2(void);
  void testAndAssign(void);
  template<unsigned int n, unsigned int m>
    void testInvertLevel3(void);
  template<unsigned int n>
    void testInvertLevel2(void);
  void testInvert(void);
  template<unsigned int n, unsigned int m>
    void testOrAssignLevel3(void);
  template<unsigned int n>
    void testOrAssignLevel2(void);
  void testOrAssign(void);
  template<unsigned int n, unsigned int m>
    void testXorAssignLevel3(void);
  template<unsigned int n>
    void testXorAssignLevel2(void);
  void testXorAssign(void);
  template<unsigned int n, unsigned int m>
    void testAndAssignInvertLevel3(void);
  template<unsigned int n>
    void testAndAssignInvertLevel2(void);
  void testAndAssignInvert(void);
  template<unsigned int n, unsigned int m>
    void testOrAssignInvertLevel3(void);
  template<unsigned int n>
    void testOrAssignInvertLevel2(void);
  void testOrAssignInvert(void);
  template<unsigned int n, unsigned int m>
    void testXorAssignInvertLevel3(void);
  template<unsigned int n>
    void testXorAssignInvertLevel2(void);
  void testXorAssignInvert(void);
  void testAssignAnd(void);
  void testAssignOr(void);
  void testAssignXOr(void);
  void testAndAssignAnd(void);
  void testAndAssignOr(void);
  void testAndAssignXOr(void);
  void testOrAssignAnd(void);
  void testOrAssignOr(void);
  void testOrAssignXOr(void);
  void testXorAssignAnd(void);
  void testXorAssignOr(void);
  void testXorAssignXOr(void);
  void testAssignInvertAnd(void);
  void testAssignInvertOr(void);
  void testAssignInvertXOr(void);
  void testAndAssignInvertAnd(void);
  void testAndAssignInvertOr(void);
  void testAndAssignInvertXOr(void);
  void testOrAssignInvertAnd(void);
  void testOrAssignInvertOr(void);
  void testOrAssignInvertXOr(void);
  void testXorAssignInvertAnd(void);
  void testXorAssignInvertOr(void);
  void testXorAssignInvertXOr(void);
  void testAssignAndInvert(void);
  void testAssignOrInvert(void);
  void testAssignXOrInvert(void);
  void testAndAssignAndInvert(void);
  void testAndAssignOrInvert(void);
  void testAndAssignXOrInvert(void);
  void testOrAssignAndInvert(void);
  void testOrAssignOrInvert(void);
  void testOrAssignXOrInvert(void);
  void testXorAssignAndInvert(void);
  void testXorAssignOrInvert(void);
  void testXorAssignXOrInvert(void);
  void testAssignInvertAndInvert(void);
  void testAssignInvertOrInvert(void);
  void testAssignInvertXOrInvert(void);
  void testAndAssignInvertAndInvert(void);
  void testAndAssignInvertOrInvert(void);
  void testAndAssignInvertXOrInvert(void);
  void testOrAssignInvertAndInvert(void);
  void testOrAssignInvertOrInvert(void);
  void testOrAssignInvertXOrInvert(void);
  void testXorAssignInvertAndInvert(void);
  void testXorAssignInvertOrInvert(void);
  void testXorAssignInvertXOrInvert(void);
  template<unsigned int n, unsigned int b>
    void testBitOpsLevel3(void);
  template<unsigned int n>
    void testBitOpsLevel2(void);
  void testBitOps(void);
  template<unsigned int n>
    void testBitAllLevel2(void);
  void testBitAll(void);
  // This is split up into multiple functions to allow parallel compilation.
  void testShift(void) { testShift_cpu1(); testShift_cpu2(); testShift_cpu3(); testShift_cpu4(); testShift_cpu5(); }
  void testShift_cpu1(void);
  void testShift_cpu2(void);
  void testShift_cpu3(void);
  void testShift_cpu4(void);
  void testShift_cpu5(void);
  // This is split up into multiple functions to allow parallel compilation.
  void testRotate(void) { testRotate_cpu1(); testRotate_cpu2(); testRotate_cpu3(); testRotate_cpu4(); testRotate_cpu5(); testRotate_cpu6(); }
  void testRotate_cpu1(void);
  void testRotate_cpu2(void);
  void testRotate_cpu3(void);
  void testRotate_cpu4(void);
  void testRotate_cpu5(void);
  void testRotate_cpu6(void);
  template<unsigned int n>
    void testIteratorLevel2(void);
  void testIterator(void);

  CPPUNIT_TEST_SUITE( bitsetTest );

  CPPUNIT_TEST( testConsts );
  CPPUNIT_TEST( testOutput );
  CPPUNIT_TEST( testConstruction );
  CPPUNIT_TEST( testEquality );
  CPPUNIT_TEST( testInequality );
  CPPUNIT_TEST( testAssignment );
  CPPUNIT_TEST( testCopy );
  CPPUNIT_TEST( testAndAssign );
  CPPUNIT_TEST( testInvert );
  CPPUNIT_TEST( testOrAssign );
  CPPUNIT_TEST( testXorAssign );
  CPPUNIT_TEST( testAndAssignInvert );
  CPPUNIT_TEST( testOrAssignInvert );
  CPPUNIT_TEST( testXorAssignInvert );
  CPPUNIT_TEST( testAssignAnd );
  CPPUNIT_TEST( testAssignAndInvert );
  CPPUNIT_TEST( testAssignInvertAnd );
  CPPUNIT_TEST( testAssignInvertAndInvert );
  CPPUNIT_TEST( testAssignOr );
  CPPUNIT_TEST( testAssignOrInvert );
  CPPUNIT_TEST( testAssignInvertOr );
  CPPUNIT_TEST( testAssignInvertOrInvert );
  CPPUNIT_TEST( testAssignXOr );
  CPPUNIT_TEST( testAssignXOrInvert );
  CPPUNIT_TEST( testAssignInvertXOr );
  CPPUNIT_TEST( testAssignInvertXOrInvert );
  CPPUNIT_TEST( testAndAssignAnd );
  CPPUNIT_TEST( testAndAssignAndInvert );
  CPPUNIT_TEST( testAndAssignInvertAnd );
  CPPUNIT_TEST( testAndAssignInvertAndInvert );
  CPPUNIT_TEST( testAndAssignOr );
  CPPUNIT_TEST( testAndAssignOrInvert );
  CPPUNIT_TEST( testAndAssignInvertOr );
  CPPUNIT_TEST( testAndAssignInvertOrInvert );
  CPPUNIT_TEST( testAndAssignXOr );
  CPPUNIT_TEST( testAndAssignXOrInvert );
  CPPUNIT_TEST( testAndAssignInvertXOr );
  CPPUNIT_TEST( testAndAssignInvertXOrInvert );
  CPPUNIT_TEST( testOrAssignAnd );
  CPPUNIT_TEST( testOrAssignAndInvert );
  CPPUNIT_TEST( testOrAssignInvertAnd );
  CPPUNIT_TEST( testOrAssignInvertAndInvert );
  CPPUNIT_TEST( testOrAssignOr );
  CPPUNIT_TEST( testOrAssignOrInvert );
  CPPUNIT_TEST( testOrAssignInvertOr );
  CPPUNIT_TEST( testOrAssignInvertOrInvert );
  CPPUNIT_TEST( testOrAssignXOr );
  CPPUNIT_TEST( testOrAssignXOrInvert );
  CPPUNIT_TEST( testOrAssignInvertXOr );
  CPPUNIT_TEST( testOrAssignInvertXOrInvert );
  CPPUNIT_TEST( testXorAssignAnd );
  CPPUNIT_TEST( testXorAssignAndInvert );
  CPPUNIT_TEST( testXorAssignInvertAnd );
  CPPUNIT_TEST( testXorAssignInvertAndInvert );
  CPPUNIT_TEST( testXorAssignOr );
  CPPUNIT_TEST( testXorAssignOrInvert );
  CPPUNIT_TEST( testXorAssignInvertOr );
  CPPUNIT_TEST( testXorAssignInvertOrInvert );
  CPPUNIT_TEST( testXorAssignXOr );
  CPPUNIT_TEST( testXorAssignXOrInvert );
  CPPUNIT_TEST( testXorAssignInvertXOr );
  CPPUNIT_TEST( testXorAssignInvertXOrInvert );
  CPPUNIT_TEST( testBitOps );
  CPPUNIT_TEST( testBitAll );
  CPPUNIT_TEST( testShift );
  CPPUNIT_TEST( testRotate );
  CPPUNIT_TEST( testIterator );

  CPPUNIT_TEST_SUITE_END();
};
