#ifndef TABLEDECODER_H
#define TABLEDECODER_H

#include <gmpxx.h>
#include <vector>

class factor_ct;

void initialize_tables(void);
void list_factors_minus(std::vector<factor_ct>& output, unsigned int n);

// The <factors> string will exist of a dot seperated list of large numbers,
// optionally ending on "P<digits>" or "[<spaces>]C<digits>".  The numbers
// can end on a star ('*').

enum factor_type_nt {
  plain_factor,			// Factor was given as a large number.
  large_prime,			// Factor was given as P<digits>.
  composite			// Factor was given as C<digits>.
};

class factor_ct {
  private:
    mpz_class M_factor;		// The real factor.
    factor_type_nt M_type;	// plain_factor, large_prime or composite.
    bool M_ended_on_star;	// The number ended on a '*'.
  public:
    // Constructors.
    factor_ct() { }
    factor_ct(mpz_class factor, factor_type_nt type, bool ended_on_star) :
        M_factor(factor), M_type(type), M_ended_on_star(ended_on_star) { }
    factor_ct(factor_ct const& factor) :
        M_factor(factor.M_factor), M_type(factor.M_type), M_ended_on_star(factor.M_ended_on_star) { }
    factor_ct& operator=(factor_ct const& factor)
        { M_factor = factor.M_factor; M_type = factor.M_type; M_ended_on_star = factor.M_ended_on_star; return *this; }
    // Accessors.
    mpz_class get_factor(void) const { return M_factor; }
    factor_type_nt get_type(void) const { return M_type; }
    bool get_ended_on_star(void) const { return M_ended_on_star; }
    // Operators.
    friend bool operator==(factor_ct const& factor1, factor_ct const& factor2)
	{ return (factor1.M_factor == factor2.M_factor); }
    friend bool operator!=(factor_ct const& factor1, factor_ct const& factor2)
	{ return (factor1.M_factor != factor2.M_factor); }
    friend bool operator<(factor_ct const& factor1, factor_ct const& factor2)
        { return (factor1.M_factor < factor2.M_factor); }
    friend bool operator>(factor_ct const& factor1, factor_ct const& factor2)
        { return (factor1.M_factor > factor2.M_factor); }
    friend bool operator<=(factor_ct const& factor1, factor_ct const& factor2)
        { return (factor1.M_factor <= factor2.M_factor); }
    friend bool operator>=(factor_ct const& factor1, factor_ct const& factor2)
        { return (factor1.M_factor >= factor2.M_factor); }
};

#endif // TABLEDECODER_H
