#include <vector>
#include <gmpxx.h>
#include <libecc/polynomial.h>
#include "mk.h"

extern void initialize_utils(void);
extern unsigned long gcd(unsigned long x, unsigned long y);
extern void factorize(unsigned long n, std::vector<unsigned long>& product);
extern mpz_class foc_reference(unsigned int n);
extern mpz_class const& foc(unsigned int n);
extern void get_divisors(unsigned long n, std::vector<unsigned long>& divisors);
extern unsigned long euler_phi(unsigned long n);
extern void euler_phi_inv(unsigned long m, std::vector<unsigned long>& result);
extern unsigned long num_normal(unsigned long n);
extern unsigned long ect1(unsigned int m);

extern int number_of_primes;
extern unsigned int primes[];
extern std::vector<unsigned long> factors_of_q_minus_one;

inline unsigned int lcm(unsigned int x, unsigned int y)
{
  return (x / gcd(x, y)) * y;
}

static unsigned int const max_m = 31;	// The largest possible m. This size is used for tables.
					// The actual largest value here is 58 (larger values cause
					// the prime table to be become too large).

static unsigned int const m = libecc::field_extension_degree;
static unsigned int const k = libecc::reduction_polynomial_exponents<m>::k;
static unsigned int const k1 = libecc::reduction_polynomial_exponents<m>::k1;
static unsigned int const k2 = libecc::reduction_polynomial_exponents<m>::k2;
static unsigned long const q = 1UL << m;

typedef libecc::bitset<m> bitset_t;
typedef libecc::polynomial<m, k, k1, k2> poly_t;

class polynomial_square {
  libecc::bitset_digit_t buffer[poly_t::square_digits];
  public:
    void operator()(poly_t& p) { p = p.square(buffer); }
};

extern poly_t const one;

extern void print_poly_on(std::ostream& os, poly_t const& p, bool html = true);
extern poly_t pow(poly_t const& x, unsigned long n);
extern poly_t generator(void);
extern bool is_generator(poly_t const& g);
extern bool is_normal_basis(poly_t const& b);
